=== modified file 'examples/simple.cc'
--- examples/simple.cc	2011-07-14 11:32:17 +0000
+++ examples/simple.cc	2011-08-11 13:05:40 +0000
@@ -46,31 +46,19 @@
 
 int main(int argc, char *argv[])
 {
-  const char* db= "information_schema";
   const char* host= NULL;
   const char* user= NULL;
   const char* password= NULL;
-  bool mysql= false;
   in_port_t port= 0;
-  const char* query= "select table_schema, table_name from tables";
-  drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
 
   for (int c; (c = getopt(argc, argv, "d:h:mp:u:P:q:v")) != -1; )
   {
     switch (c)
     {
-    case 'd':
-      db= optarg;
-      break;
-
     case 'h':
       host= optarg;
       break;
 
-    case 'm':
-      mysql= true;
-      break;
-
     case 'p':
       port= static_cast<in_port_t>(atoi(optarg));
       break;
@@ -83,40 +71,27 @@
       password = optarg;
       break;
 
-    case 'q':
-      query= optarg;
-      break;
-
-    case 'v':
-      if (verbose < DRIZZLE_VERBOSE_MAX)
-        verbose= static_cast<drizzle_verbose_t>(verbose + 1);
-      break;
-
     default:
       cout << 
-        "usage: " << argv[0] << " [-d <db>] [-h <host>] [-m] [-p <port>] [-q <query>] [-v]\n"
-        "\t-d <db>    - Database to use for query\n"
+        "usage:\n"
         "\t-h <host>  - Host to connect to\n"
-        "\t-m         - Use the MySQL protocol\n"
         "\t-p <port>  - Port to connect to\n"
         "\t-u <user>  - User\n"
-        "\t-P <pass>  - Password\n"
-        "\t-q <query> - Query to run\n"
-        "\t-v         - Increase verbosity level\n";
+        "\t-P <pass>  - Password\n";
       return 1;
     }
   }
 
   drizzle::drizzle_c drizzle;
-  drizzle_set_verbose(&drizzle.b_, verbose);
   drizzle::connection_c* con= new drizzle::connection_c(drizzle);
-  if (mysql)
-    drizzle_con_add_options(&con->b_, DRIZZLE_CON_MYSQL);
   con->set_tcp(host, port);
   con->set_auth(user, password);
-  con->set_db(db);
+  con->set_db("information_schema");
   drizzle::result_c result;
-  if (con->query(result, query))
+  drizzle::query_c q(*con, "select table_schema, table_name from tables where table_name like ?");
+  q.p("%");
+  cout << q.read() << endl;
+  if (q.execute(result))
   {
     cerr << "query: " << con->error() << endl;
     return 1;

=== modified file 'libdrizzle-2.0/libdrizzle/libdrizzle.hpp'
--- libdrizzle-2.0/libdrizzle/libdrizzle.hpp	2011-07-14 11:23:33 +0000
+++ libdrizzle-2.0/libdrizzle/libdrizzle.hpp	2011-08-11 13:51:18 +0000
@@ -35,6 +35,7 @@
 
 #include <cstring>
 #include <libdrizzle/libdrizzle.h>
+#include <sstream>
 #include <stdexcept>
 
 namespace drizzle {
@@ -43,7 +44,18 @@
 {
 };
 
-class drizzle_c
+class noncopyable
+{
+protected:
+  noncopyable()
+  {
+  }
+private:
+  noncopyable(const noncopyable&);
+  void operator=(const noncopyable&);
+};
+
+class drizzle_c : noncopyable
 {
 public:
   drizzle_c()
@@ -59,7 +71,7 @@
   drizzle_st b_;
 };
 
-class result_c
+class result_c : noncopyable
 {
 public:
   result_c()
@@ -120,7 +132,7 @@
   drizzle_result_st b_;
 };
 
-class connection_c
+class connection_c : noncopyable
 {
 public:
   explicit connection_c(drizzle_c& drizzle)
@@ -175,25 +187,84 @@
   drizzle_con_st b_;
 };
 
-/*
-inline drizzle_return_t query(drizzle_con_st* con, result_c& result, const char* str, size_t str_size)
-{
-  drizzle_return_t ret;
-  drizzle_query(con, &result.b_, str, str_size, &ret);
-  if (ret == DRIZZLE_RETURN_OK)
-    ret = drizzle_result_buffer(&result.b_);
-  return ret;
-}
-
-inline drizzle_return_t query(drizzle_con_st* con, result_c& result, const std::string& str)
-{
-  return query(con, result, str.data(), str.size());
-}
-
-inline drizzle_return_t query(drizzle_con_st* con, result_c& result, const char* str)
-{
-  return query(con, result, str, strlen(str));
-}
-*/
+class query_c
+{
+public:
+  query_c(connection_c& con, const std::string& in = "") :
+    con_(con),
+    in_(in)
+  {
+  }
+
+  void operator=(const std::string& v)
+  {
+    in_ = v;
+    out_.clear();
+  }
+
+  void operator+=(const std::string& v)
+  {
+    in_ += v;
+  }
+
+  query_c& p_name(const std::string& v)
+  {
+    std::vector<char> r(2 * v.size() + 2);
+    r.resize(drizzle_escape_string(&r.front() + 1, v.data(), v.size()) + 2);    
+    r.front() = '`';
+    r.back() = '`';
+    p_raw(&r.front(), r.size());
+    return *this;
+  }
+
+  query_c& p_raw(const char* v, size_t sz)
+  {
+    size_t i = in_.find('?');
+    assert(i != std::string::npos);
+    if (i == std::string::npos)
+      return *this;
+    out_.append(in_.substr(0, i));
+    in_.erase(0, i + 1);
+    out_.append(v, sz);
+    return *this;
+  }
+
+  query_c& p_raw(const std::string& v)
+  {
+    return p_raw(v.data(), v.size());
+  }
+
+  query_c& p(const std::string& v)
+  {
+    std::vector<char> r(2 * v.size() + 2);
+    r.resize(drizzle_escape_string(&r.front() + 1, v.data(), v.size()) + 2);    
+    r.front() = '\'';
+    r.back() = '\'';
+    p_raw(&r.front(), r.size());
+    return *this;
+  }
+
+  query_c& p(long long v)
+  {
+    std::stringstream ss;
+    ss << v;
+    p_raw(ss.str());
+    return *this;
+  }
+
+  drizzle_return_t execute(result_c& result)
+  {
+    return con_.query(result, read());
+  }
+
+  std::string read() const
+  {
+    return out_ + in_;
+  }
+private:
+  connection_c& con_;
+  std::string in_;
+  std::string out_;
+};
 
 }

